#include "slm_manual.h"
#include "slm_manual_keys.h"
#include "slm_driver.h"
#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <curses.h>
#include <g++-3/iostream.h>
#include <stdlib.h>

main(int argc,char ** argv){

// set initial values

float x_offset=0;
float focal_length=2;
float line_spacing=37.5;
float slm_size=1.6;
float distortion_factor=1.2;
float ms_ratio=0.5;
int frame_rate=1;

float factor_x_offset=100;
float factor_focal_length=0.1;
float factor_line_spacing=1;

int quit=FALSE;
int update_slm=FALSE;
int first_run=TRUE;
char keypress;
char kp2;
char inptvalue[12];
float pixel_size;

int tmp;

float line_spacing_p, focal_length_p, x_offset_p;

	if(argc!=1){
		cout<<"No arguments!\n";
		exit(3);
		}
	initscr();
	cbreak();
	noecho();
	echo();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	refresh();

	cout<<"Steve's manual arc manouverer.\r\n"<<flush;
	cout<<"Press `h' for help, or `q' for quit, `a' to show the arc attributes.\r\n"<<flush;
	SLM_reset();
	SLM_repeat(25000);


	// now work out the conversion factors from real world units into SLM pixels

	pixel_size=(slm_size*1000)/256; //in microns
	x_offset_p=128-(x_offset/pixel_size);
	focal_length_p=(((slm_size/2)-focal_length)*1000)/pixel_size;
	line_spacing_p=line_spacing/pixel_size;



	do{
		keypress=getchar();
		switch(keypress){
			case LINE_SPACING_PLUS:
				line_spacing+=factor_line_spacing;
				cout<<"Line spacing: "<<line_spacing<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case LINE_SPACING_MINUS:
				line_spacing-=factor_line_spacing;
				cout<<"Line spacing: "<<line_spacing<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_PLUS:
				focal_length+=factor_focal_length;
				cout<<"Mean focal length: "<<focal_length<<" mm\r"<<endl;
				update_slm=TRUE;
				break;

			case FOCAL_LENGTH_MINUS:
				focal_length-=factor_focal_length;
				cout<<"Mean focal length: "<<focal_length<<" mm\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_PLUS:
				x_offset+=factor_x_offset;
				cout<<"x offset: "<<x_offset<<" microns\r"<<endl;
				update_slm=TRUE;
				break;

			case X_OFFSET_MINUS:
				x_offset-=factor_x_offset;
				cout<<"x offset: "<<x_offset<<" microns\r"<<endl;
				update_slm=TRUE;
				break;


			case SET_TO:
				cout<<"Set "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING:
						cout<<"line spacing.\r\nCurrent line spacing: "<<line_spacing<<" microns. Enter new line spacing: "<<flush;
						echo();
						getnstr(inptvalue,10);line_spacing=strtod(inptvalue,NULL);
						noecho();
						break;
					case FOCAL_LENGTH:
						cout<<"mean focal length.\r\nCurrent mean focal length: "<<focal_length<<" mm. Enter new mean focal length: "<<flush;
						echo();
						getnstr(inptvalue,10);focal_length=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset: "<<x_offset<<" microns. Enter new x offset: "<<flush;
						echo();
						getnstr(inptvalue,10);x_offset=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case SLM_SIZE:
						cout<<"image size.\r\nCurrent image size: "<<slm_size<<" mm. Enter new image size: "<<flush;
						echo();
						getnstr(inptvalue,10);slm_size=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case DISTORTION_FACTOR:
						cout<<"distortion factor.\r\nCurrent distortion factor: "<<distortion_factor<<". Enter distortion factor: "<<flush;
						echo();
						getnstr(inptvalue,10);distortion_factor=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case REP_RATE:
						cout<<"frame repetition rate.\r\nCurrent frame repetition rate: "<<frame_rate<<" Hz. Enter new repetition rate (integer): "<<flush;
						echo();
						getnstr(inptvalue,10);frame_rate=strtol(inptvalue,NULL,10);
						cout<<"\r"<<flush;
						noecho();
						update_slm=TRUE;
						break;
					case MS_RATIO:
						cout<<"mark:space ratio.\r\nCurrent mark:space ratio: "<<ms_ratio<<" Enter new mark:space ratio: "<<flush;
						echo();
						getnstr(inptvalue,10);ms_ratio=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						update_slm=TRUE;
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				update_slm=TRUE;
				break;

			case FACTOR:
				cout<<"Set factor: "<<flush;
				kp2=getchar();
				switch(kp2){
					case LINE_SPACING:
						cout<<"line spacing.\r\nCurrent line spacing factor: "<<factor_line_spacing<<" microns. Enter new line spacing factor: "<<flush;
						echo();
						getnstr(inptvalue,10);factor_line_spacing=strtod(inptvalue,NULL);
						noecho();
						cout<<"\r"<<flush;
						break;
					case FOCAL_LENGTH:
						cout<<"focal length.\r\nCurrent mean focal length factor: "<<factor_focal_length<<" mm. Enter new mean focal length factor: "<<flush;
						echo();
						getnstr(inptvalue,10);factor_focal_length=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					case X_OFFSET:
						cout<<"x offset.\r\nCurrent x offset factor: "<<factor_x_offset<<" microns. Enter new x offset factor: "<<flush;
						echo();
						getnstr(inptvalue,10);factor_x_offset=strtod(inptvalue,NULL);
						cout<<"\r"<<flush;
						noecho();
						break;
					default:
						cout<<"Nothing changed.\n";
						cout<<"\r"<<endl;
						break;
					}
				break;

			case QUIT:
				quit=TRUE;
				break;
			case ATTRIBUTES:
				if(first_run==TRUE){
					cout<<"Cannot determine attributes, since they have not been set yet.\r"<<endl;
					cout<<"Press `d' to set them to defaults.\r"<<endl<<endl;
					}
				else{
					cout<<"The arcs have the following characteristics:\r"<<endl;
					cout<<"Line spacing:      "<<line_spacing<<" microns\r"<<endl;
					cout<<"Mean focal length: "<<focal_length<<" mm\r"<<endl;
					cout<<"x offset:          "<<x_offset<<" microns\r"<<endl;
					cout<<"Mark:space ratio:  "<<ms_ratio<<"\r"<<endl<<endl;
					cout<<"Current scale factors:\r"<<endl;
					cout<<"Line spacing factor:      "<<factor_line_spacing<<" microns\r"<<endl;
					cout<<"Mean focal length factor: "<<factor_focal_length<<" mm\r"<<endl;
					cout<<"x offset factor:          "<<factor_x_offset<<" microns\r"<<endl<<endl;
					cout<<"SLM image size:    "<<slm_size<<" mm\r"<<endl;
					cout<<"Distortion factor: "<<distortion_factor<<"\r"<<endl;
					cout<<"Frame repetition rate: "<<frame_rate<<"\r"<<endl<<endl;
					}
				break;
			case HELP:
				cout<<endl<<"Use these keys to adjust the arc attributes by the current scale factors:\r"<<endl;
				cout<<"   + -   increase/decrease line spacing\r"<<endl;
				cout<<"   v ^   increase/decrease mean focal length\r"<<endl;
				cout<<"   < >   move x offset to the left/right\r"<<endl<<endl;
				cout<<"To set the values absolutely, press `s' followed by:\r"<<endl;
				cout<<"    l    line spacing\r"<<endl;
				cout<<"    f    mean focal length\r"<<endl;
				cout<<"    x    x offset\r"<<endl;
				cout<<"    s    SLM size\r"<<endl;
				cout<<"    d    distortion factor\r"<<endl;
				cout<<"    m    mark:space ratio (actually the portion of light that is 'off')\r"<<endl;
				cout<<"    r    frame repetition rate\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"To change the scale factor, press `f' followed by:\r"<<endl;
				cout<<"    l    line spacing factor\r"<<endl;
				cout<<"    f    mean focal length factor\r"<<endl;
				cout<<"    x    x offset factor\r"<<endl;
				cout<<"                     ... and then enter the value\r"<<endl<<endl;
				cout<<"Press `q' to quit, `a' to show the arc attributes, `d' for defaults.\r"<<endl;
				break;
			case DEFAULT:
				cout<<"Arc attributes, scales factors etc have been reset to default.\r"<<endl;
				cout<<"Press `a' to find out what they are now.\r"<<endl<<endl;

				x_offset=0;
				focal_length=2;
				line_spacing=37.5;
				slm_size=1.6;
				distortion_factor=1.2;
				ms_ratio=0.5;
				frame_rate=1;
				factor_x_offset=100;
				factor_focal_length=0.1;
				factor_line_spacing=1;

				update_slm=TRUE;
				break;
			case RETURN:
				cout<<"\r"<<endl;
				break;
			default:
				break;
			}


		// now work out the conversion factors from real world units into SLM pixels

		pixel_size=(slm_size*1000)/256; //in microns
		x_offset_p=128-(x_offset/pixel_size);
		focal_length_p=(((slm_size/2)-focal_length)*1000)/pixel_size;
		line_spacing_p=line_spacing/pixel_size;

		if(update_slm==TRUE){
			SLM_upload_arcs(x_offset_p, focal_length_p, line_spacing_p, distortion_factor, ms_ratio, frame_rate);
			update_slm=FALSE;
			first_run=FALSE;
			}		
		}while(quit!=TRUE);
		
	nocbreak(); echo(); endwin();
	}



void SLM_upload_arcs(float x_offset_p, float y_offset_p, float sep_p, float y_distort_sq, float ms_ratio, int frame_rate)
{
int x,y,l,b;
unsigned char fr[8192];
int frame[8192];
float cone[65536];
int arcs[65536];
short frl[3*2];

	for(x=0;x<256;x++){
		for(y=0;y<256;y++){
			cone[x+(y*256)]=sqrt((y_distort_sq*(y_offset_p-y)*(y_offset_p-y))+((x_offset_p-x)*(x_offset_p-x)));
			}
		}
	for(x=0;x<256;x++){
		for(y=0;y<256;y++){
			arcs[x+(y*256)]=(fmod(cone[y+(x*256)],sep_p) >= (sep_p*ms_ratio));
			}
		}
	for(l=0;l<8192;l++){
		frame[l]=0;
		}

	for(l=0;l<8192;l++){
		for(b=0;b<8;b++){
			frame[l]=frame[l] | (int)(pow(2,b)*(arcs[(l*8)+b]!=0));
			fr[l]=(uchar)frame[l];
			}
		}

//	cout<<"Uploading to frame "<<1<<"\n";
	SLM_upload_frame(1,fr);
	SLM_timer('H');
//	cout<<"Setting FRL\n";
	frl[0]=1;frl[1]=0;frl[2]=1;
	frl[3]=1;frl[4]=0x8000;frl[5]=1;
	SLM_upload_FRL(2,frl);
	SLM_timer_rate((short)frame_rate);
	SLM_timer('G');
	}
