// SLM driver for unix matt november 2000
// after Carl Paterson IC
// after Copyright (c) 1996 by Displaytech, Inc.  2200 Central Ave.  Boulder, CO 
#ifndef __SLM__
#define __SLM__
//#include <stdlib.h>
#include <sys/io.h> 
//#include <ctype.h>
#include <iostream.h>

#ifndef TRUE
typedef int	BOOL;
#define TRUE	1
#define FALSE 	0
#endif

typedef unsigned char uchar;

#define SLM_GOOD	TRUE
#define SLM_BAD		FALSE
// Parallel port registers
#define LPT_DATA                0x378ul
#define LPT_STAT                0x379ul
#define LPT_CTRL                0x37Aul

#define SLM_IMAGE_SIZE		8192
#define SLM_ROW_SIZE		2048
#define SLM_ROWS_PER_IMAGE	4
#define SLM_MCU_TIMER_RATE	125000

#define SLM_LPT_STAT             0x379ul
// Register Select Bit definitions
#define SLM_REG0_SEL		(((0x00^5)<<1)&0xE)
#define SLM_REG1_SEL		(((0x01^5)<<1)&0xE)
#define SLM_REG2_SEL		(((0x02^5)<<1)&0xE)
#define SLM_REG3_SEL		(((0x03^5)<<1)&0xE)
#define SLM_REG4_SEL		(((0x04^5)<<1)&0xE)
#define SLM_REG5_SEL		(((0x05^5)<<1)&0xE)
#define SLM_REG6_SEL		(((0x06^5)<<1)&0xE)
#define SLM_REG7_SEL		(((0x07^5)<<1)&0xE)
// Parasite Status Registers
#define SLM_NUM_REGS		9
#define SLM_STAT_SYS		0
#define SLM_STAT_CMD		1
#define SLM_STAT_VRM		2
#define SLM_STAT_FRL		3
#define SLM_STAT_PRK		4
#define SLM_STAT_COP		5
#define SLM_STAT_CLK		6
#define SLM_STAT_SLM		7
#define SLM_STAT_WIN		8

#define SLM_BUSY()		((inb(SLM_LPT_STAT) & 0x80) == 0)
#define SLM_CMD_DONE		0x20
#define SLM_CMD_INVD		0x08
#define SLM_PRM_INVD		0x10
#define SLM_TIMEOUT		-1
#define SLM_OK			0

#define dummyCmd		0
#define SLM_NO_TRIES		65530	// Parallel Port Busy Timeout
#define SLM_error	int
#define SLM_status	int

// slm status bits
// SLM_STAT_SYS
#define SLM_STATUS_SYS_FAULT		0x08
#define SLM_STATUS_SYS_PRK		0x10
//#define SLM_NORMAL			0x00
//SLM_STAT_CMD
#define SLM_STATUS_CMD_BAD		0x08
#define SLM_STATUS_PARAM_BAD		0x10
//#define SLM_CMD_GOOD			0x00
//SLM_VRM-none defined matt 2000
//SLM_STAT_FRL
#define SLM_STATUS_FRL_BAD_PTR		0x08
#define SLM_STATUS_FRL_BAD_PAGE		0x10
//SLM_STAT_PRK
#define SLM_STATUS_PRK1			0x08
#define SLM_STATUS_PRK2			0x10
//SLM_STAT_COP--none defined matt 2000
//SLM_STAT_CLK--none defined matt 2000
//SLM_STAT_WIN
#define SLM_STATUS_SLM1_WIN_BAD		0x20
#define SLM_STATUS_SLM2_WIN_BAD		0x10
//SLM_STAT_SLM
#define	SLM_STATUS_SLM1			0x20
#define SLM_STATUS_SLM2			0x10

typedef struct SLM_slim_status_s{
	BOOL	slm1,slm2,prk1,prk2,win1,win2;
	BOOL	cmd,param,frlptr,frlpage;
	BOOL	sysprk,status;
	int	coprsts,clkrsts;
	uchar	regs[SLM_NUM_REGS];
	} SLM_slim_status_s;

// write to reg0
SLM_error	SLM_issue_cmd(uchar);

//issue a simple no parameter message
SLM_error	SLM_write(uchar);

// COMPLEX writes inlcuding the "parameter count"
// which is sometimes user defined
SLM_error	SLM_write(uchar,uchar,uchar,uchar);
SLM_error	SLM_write(uchar,uchar,uchar,uchar,uchar,uchar);
SLM_error	SLM_write(uchar,uchar,uchar,uchar,uchar,uchar,uchar,uchar);

SLM_error	SLM_write(uchar,uchar,short);
SLM_error	SLM_write(uchar,uchar,short,short);
SLM_error	SLM_write(uchar,uchar,short,short,short);

// SIMPLE writes (wrappers for COMPLEX writes) works out the parameter count for you
SLM_error	SLM_write(uchar,uchar,uchar);
SLM_error	SLM_write(uchar,uchar,uchar,uchar,uchar);
SLM_error	SLM_write(uchar,uchar,uchar,uchar,uchar,uchar,uchar);

SLM_error	SLM_write(uchar,short);
SLM_error	SLM_write(uchar,short,short);
SLM_error	SLM_write(uchar,short,short,short);


 
SLM_error 	SLM_repeat(short);
SLM_error 	SLM_reset(void);
SLM_error 	SLM_timer(uchar);
SLM_error 	SLM_timer_rate(short);
SLM_error 	SLM_upload_frame(int, uchar *data) ;
SLM_error 	SLM_upload_FRL(int, short *data) ;
SLM_error 	SLM_write_slm_parport(uchar,uchar) ;
SLM_error	SLM_wait_on_parport( void) ;
void	SLM_get_slim_status	(SLM_slim_status_s *);
SLM_status	SLM_get_status( void) ;
void		SLM_send_dummy(void) ;
void		SLM_print_status(void) ;

#endif
